# ~~~
# This file is part of the paper:
#
#           " An Online Efficient Two-Scale Reduced Basis Approach
#                for the Localized Orthogonal Decomposition "
#
#   https://github.com/TiKeil/Two-scale-RBLOD.git
#
# Copyright 2019-2021 all developers. All rights reserved.
# License: Licensed as BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
# Authors:
#   Tii1     (2018)
#   Tim Keil (2019 - 2021)
# ~~~

from __future__ import absolute_import

import os
from sys import platform, exit, stderr

if platform == 'mac':
    import MacOS
    def time():
        return MacOS.GetTicks() / 60.0
    timekind = "real"
elif hasattr(os, 'times'):
    def time():
        t = os.times()
        return t[0] + t[1]
    timekind = "cpu"
else:
    stderr.write(
        "Don't know how to get time on platform %s\n" % repr(platform))
    exit(1)
